@@ -5,7 +5,8 @@
 #'
 #' @param data List. Output of \code{oa_request}.
 #' @param entity Character. Scholarly entity of the search.
-#' The argument can be one of c("works", "authors", "venues", "institutions", "concepts").
+#' The argument can be one of
+#' c("works", "authors", "institutions", "concepts", "funders", "sources", "publishers").
 #' @param abstract Logical. If TRUE, the function returns also the abstract of each item.
 #' Ignored if entity is different from "works". Defaults to TRUE.
 #' @param verbose Logical.
@@ -55,7 +56,7 @@ oa2df <- function(data, entity, options = NULL, count_only = FALSE, group_by = N
     return(do.call(rbind.data.frame, data))
   }
 
-  if (count_only && length(data) == 4) {
+  if (count_only && length(data) < 8) { # assuming less than 8 fields in output$meta
     return(unlist(data))
   }
 
@@ -70,7 +71,6 @@ oa2df <- function(data, entity, options = NULL, count_only = FALSE, group_by = N
     works = works2df(data, abstract, verbose),
     authors = authors2df(data, verbose),
     institutions = institutions2df(data, verbose),
-    venues = venues2df(data, verbose),
     concepts = concepts2df(data, verbose),
     funders = funders2df(data, verbose),
     sources = sources2df(data, verbose),
@@ -137,18 +137,20 @@ works2df <- function(data, abstract = TRUE, verbose = TRUE,
                      pb = if (verbose) oa_progress(length(data)) else NULL) {
 
   col_order <- c(
-    "id", "display_name", "author", "ab", "publication_date", "relevance_score",
+    "id", "title", "display_name", "author", "ab", "publication_date", "relevance_score",
     "so", "so_id", "host_organization", "issn_l", "url", "pdf_url",
     "license", "version", "first_page", "last_page", "volume", "issue", "is_oa",
     "is_oa_anywhere", "oa_status", "oa_url", "any_repository_has_fulltext",
     "language", "grants", "cited_by_count", "counts_by_year",
     "publication_year", "cited_by_api_url", "ids", "doi", "type",
-    "referenced_works", "related_works", "is_paratext", "is_retracted", "concepts"
+    "referenced_works", "related_works", "is_paratext", "is_retracted",
+    "concepts", "topics"
   )
   works_process <- tibble::tribble(
     ~type, ~field,
     "identical", "id",
     "identical", "display_name",
+    "identical", "title",
     "identical", "publication_date",
     "identical", "doi",
     "identical", "type",
@@ -224,13 +226,13 @@ works2df <- function(data, abstract = TRUE, verbose = TRUE,
             first_inst <- empty_inst
           }
           first_inst <- prepend(first_inst, "institution")
-          aff_raw <- list(au_affiliation_raw = l$raw_affiliation_string[1])
+          aff_raw <- list(au_affiliation_raw = replace_w_na(l$raw_affiliation_string[1]))
           l_author <- if (length(l$author) > 0) {
             prepend(replace_w_na(l$author), "au")
           } else {
             empty_list(c("au_id", "au_display_name", "au_orcid"))
           }
-          c(l_author, l["author_position"], aff_raw, first_inst)
+          c(l_author, l[c("author_position", "is_corresponding")], aff_raw, first_inst)
         }), "rbind_df"
       )
     }
@@ -245,7 +247,31 @@ works2df <- function(data, abstract = TRUE, verbose = TRUE,
       names(open_access)[[1]] <- "is_oa_anywhere"
     }
 
-    out_ls <- c(sim_fields, venue, open_access, paper_biblio, list(author = author, ab = ab))
+    # Topics
+    process_paper_topics <- function(paper) {
+      topics <- paper$topics
+      if (is.null(topics)) return(NULL)
+      topics_ls <- lapply(seq_along(topics), function(i) {
+        topic <- topics[[i]]
+        relev <- c(
+          # Hoist fields for the topic entity
+          list(topic = topic[c("id", "display_name")]),
+          # Keep info about other entities as-is
+          topic[vapply(topic, is.list, logical(1))]
+        )
+        relev_df <- subs_na(relev, "rbind_df")[[1]]
+        relev_df <- tibble::rownames_to_column(relev_df, "name")
+        cbind(i = i, score = topic$score, relev_df)
+      })
+      topics_df <- do.call(rbind.data.frame, topics_ls)
+      list(tibble::as_tibble(topics_df))
+    }
+    topics <- process_paper_topics(paper)
+
+    out_ls <- c(
+      sim_fields, venue, open_access, paper_biblio,
+      list(author = author, ab = ab, topics = topics)
+    )
     out_ls[sapply(out_ls, is.null)] <- NULL
     list_df[[i]] <- out_ls
   }
@@ -292,8 +318,8 @@ abstract_build <- function(ab) {
 #' query_author <- oa_query(
 #'   identifier = NULL,
 #'   entity = "authors",
-#'   last_known_institution.id = "I71267560",
-#'   works_count = ">99"
+#'   last_known_institutions.id = "I71267560",
+#'   works_count = ">500"
 #' )
 #'
 #' res <- oa_request(
@@ -315,6 +341,7 @@ authors2df <- function(data, verbose = TRUE,
 
   inst_cols <- c("id", "display_name", "ror", "country_code", "type", "lineage")
   empty_inst <- empty_list(inst_cols)
+  empty_inst$affiliations_other <- list(NULL)
 
   author_process <- tibble::tribble(
     ~type, ~field,
@@ -343,19 +370,25 @@ authors2df <- function(data, verbose = TRUE,
       fields$type,
       SIMPLIFY = FALSE
     )
-
-    sub_affiliation <- item$last_known_institution
-    if (!is.null(sub_affiliation)) {
+    sub_affiliation <- item$last_known_institutions
+    if (!is.null(sub_affiliation) && length(sub_affiliation)) {
+      sub_affiliation <- sub_affiliation[[1]]
       if (is.na(sub_affiliation[[1]])) {
         sub_affiliation <- empty_inst
       }
-      if (length(sub_affiliation$lineage) > 1) {
-        sub_affiliation$lineage <- paste(sub_affiliation$lineage, collapse = ", ")
-      }
+      sub_affiliation$lineage <- paste(sub_affiliation$lineage, collapse = ", ")
       sub_affiliation <- prepend(sub_affiliation, "affiliation")
     }
     sub_affiliation <- replace_w_na(sub_affiliation)
 
+    if (!is.null(item$affiliations)) {
+      affiliations_other <- sapply(item$affiliations, function(x) x$institution$id)
+      if (!is.null(sub_affiliation$affiliation_id)) {
+        affiliations_other <- affiliations_other[affiliations_other != sub_affiliation$affiliation_id]
+      }
+      sub_affiliation$affiliations_other <- list(affiliations_other)
+    }
+
     list_df[[i]] <- c(sim_fields, sub_affiliation)
   }
 
@@ -364,6 +397,7 @@ authors2df <- function(data, verbose = TRUE,
     "ids", "orcid", "works_count", "cited_by_count", "counts_by_year",
     "affiliation_display_name", "affiliation_id", "affiliation_ror",
     "affiliation_country_code", "affiliation_type", "affiliation_lineage",
+    "affiliations_other",
     "x_concepts", "works_api_url"
   )
 
@@ -473,92 +507,6 @@ institutions2df <- function(data, verbose = TRUE,
 }
 
 
-#' Convert OpenAlex collection of venues' records from list format to data frame
-#'
-#' It converts bibliographic collection of venues' records gathered from OpenAlex database \href{https://openalex.org/}{https://openalex.org/} into data frame.
-#' The function converts a list of venues' records obtained using \code{oa_request} into a data frame/tibble.
-#'
-#' @inheritParams works2df
-#'
-#' @return a data.frame.
-#'
-#' For more extensive information about OpenAlex API, please visit: <https://docs.openalex.org>
-#'
-#'
-#' @examples
-#' \dontrun{
-#'
-#' # Query to search information about the Journal of Informetrics (OA id:V205292342)
-#'
-#'
-#' query_inst <- oa_query(
-#'   identifier = "V205292342",
-#'   entity = "venues"
-#' )
-#'
-#' res <- oa_request(
-#'   query_url = query_inst,
-#'   count_only = FALSE,
-#'   verbose = FALSE
-#' )
-#'
-#' df <- oa2df(res, entity = "venues")
-#'
-#' df
-#' }
-#'
-#' @export
-venues2df <- function(data, verbose = TRUE,
-                      pb = if (verbose) oa_progress(length(data)) else NULL) {
-
-  n <- length(data)
-  list_df <- vector(mode = "list", length = n)
-  venue_process <- tibble::tribble(
-    ~type, ~field,
-    "identical", "id",
-    "identical", "display_name",
-    "identical", "host_organization_name",
-    "identical", "works_count",
-    "identical", "cited_by_count",
-    "identical", "is_oa",
-    "identical", "is_in_doaj",
-    "identical", "homepage_url",
-    "identical", "works_api_url",
-    "identical", "type",
-    "identical", "relevance_score",
-    "flat", "issn_l",
-    "flat", "issn",
-    "rbind_df", "counts_by_year",
-    "rbind_df", "x_concepts",
-    "flat", "ids"
-  )
-
-  for (i in seq.int(n)) {
-    if (verbose) pb$tick()
-
-    item <- data[[i]]
-
-    fields <- venue_process[venue_process$field %in% names(item), ]
-    sim_fields <- mapply(
-      function(x, y) subs_na(item[[x]], type = y),
-      fields$field,
-      fields$type,
-      SIMPLIFY = FALSE
-    )
-    list_df[[i]] <- sim_fields
-  }
-
-  col_order <- c(
-    "id", "display_name", "host_organization_name", "issn", "issn_l", "is_oa", "is_in_doaj",
-    "ids", "homepage_url", "relevance_score", "works_count", "cited_by_count",
-    "counts_by_year", "x_concepts", "works_api_url", "type"
-  )
-
-  out_df <- rbind_oa_ls(list_df)
-  out_df[, intersect(col_order, names(out_df))]
-}
-
-
 #' Convert OpenAlex collection of concepts' records from list format to data frame
 #'
 #' It converts bibliographic collection of concepts' records gathered from OpenAlex database \href{https://openalex.org/}{https://openalex.org/} into data frame.
@@ -779,7 +727,7 @@ sources2df <- function(data, verbose = TRUE,
     "identical", "is_in_doaj",
     "flat", "ids",
     "identical", "homepage_url",
-    "identical", "apc_prices",
+    "rbind_df", "apc_prices",
     "identical", "apc_usd",
     "identical", "country_code",
     "flat", "societies",
